<?php

function MySQLCreate()
{
    //NeoLog( 3, "DB_HOSTNAME: " . DB_HOSTNAME );
    //NeoLog( 3, "DB_USERNAME: " . DB_USERNAME );
    //NeoLog( 3, "DB_PASSWORD: " . DB_PASSWORD );
    //NeoLog( 3, "SSL_BASE:    " . SSL_BASE    );

    $m = mysqli_init();

    if ( !$m )
    {
        NeoLog( "Could not initialise 'mysqli'" );
        exit;
    }

    ConfigureSSL( $m, SSL_BASE, DB_HOSTNAME );

    $connection = @$m->real_connect( DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, null, null, null, MYSQLI_CLIENT_SSL_DONT_VERIFY_SERVER_CERT );

    if ( !$connection )
    {
        if ( "No such file or directory" == $m->error )
        {
            NeoLog( "Aborting: could not connect to " . DB_HOSTNAME . " using " . DB_USERNAME . ":" . DB_PASSWORD );
            exit;
        }
        else
        {
            NeoLog( "Aborting: unexpected error: " . $m->error );
            exit;
        }
    }

    return $m; // Same as $m.
}

function MySQLClose( $m )
{
    mysqli_close( $m );
}

function MySQLInfo( $mysql, $sql_query )
{
    $tuples   = array();
    $resource = mysqli_query( $mysql, $sql_query );

    if ( !$resource )
    {
        NeoLog( $mysql->error );
        exit;
    }
    else
    {
        while ( $row = mysqli_fetch_array( $resource, MYSQLI_ASSOC ) )
        {
            $tuples[] = $row;
        }
    }
    return $tuples;
}

function MySQLFunction( $db, $function )
{
    $ret      = null;
    $sql      = "SELECT $function";
    $m        = MySQLCreate();

    if ( ! mysqli_select_db( $m, $db ) )
    {
        NeoLog( "Aborting, could not select database! " . $db );
        exit;
    }
    else
    {
        $resource = mysqli_query( $m, $sql );

        if ( True === $resource )
        {
            $ret = True;
        }
        else
        if( False === $resource )
        {
            $ret = False;
        }
        else
        if ( $resource )
        {
            $row = mysqli_fetch_array( $resource, MYSQLI_NUM );
            $ret = array_key_exists( 0, $row ) ? $row[0] : NULL;
            mysqli_free_result( $resource );
        }
        else
        {
            NeoLog( "Aborting, " . $m->error );
            exit;
        }
        mysqli_next_result( $m );
    }

    MySQLClose( $m );

    return $ret;
}

function MySQLProcedure( $db, $sp_call, &$procedure = "" )
{
    $ret = False;

    if ( !is_object( $sp_call ) )
    {
        debug_backtrace();
    }

    $proc_name = $sp_call->proc_name;
    $arguments = $sp_call->arguments;

    $m = MySQLCreate();

    if ( ! mysqli_select_db( $m, $db ) )
    {
        NeoLog( "Aborting, could not select database! " . $db );
        MySQLClose( $m );
        exit;
    }
    else
    {
        $filtered = array();
        foreach( $arguments as $argument )
        {
            if ( is_null( $argument ) )
            {
                $filtered[] = "NULL";
            }
            else
            {
                $filtered[] = "'" . mysqli_real_escape_string( $m, $argument ) . "'";
            }
        }

        $procedure = $proc_name . "(" . join( ",", $filtered ) . ")";

        $sql_query = "CALL " . $procedure;

        if ( "wab_test" == $proc_name )
        {
            NeoLog( "Test: " . $sql_query );
        }

        $resource  = mysqli_query( $m, $sql_query );

        if ( True === $resource )
        {
            $ret = array();
        }
        else
        if( False === $resource )
        {
            $ret = $m->error;
        }
        else
        if ( $resource )
        {
            $ret = array();

            while ( $row = mysqli_fetch_array( $resource, MYSQLI_ASSOC ) )
            {
                $ret[] = (object) $row;
            }

            mysqli_free_result( $resource );
        }
        else
        {
            NeoLog( "Aborting, " . $m->error );
            MySQLClose( $m );
            exit;
        }
        mysqli_next_result( $m );
    }

    MySQLClose( $m );

    return $ret;
}

class ResultIterator
{
    protected $m;
    protected $result;
    protected $row;

    function __construct( $anM, $aResult )
    {
        $this->m      = $anM;
        $this->result = $aResult;
        $this->row    = mysqli_fetch_array( $this->result, MYSQLI_ASSOC );
        $this->first  = $this->row;
    }

    function __destruct()
    {
        mysqli_free_result( $this->result );
        mysqli_next_result( $this->m      );
        MySQLClose        ( $this->m      );
    }

    function hasFirst()
    {
        return $this->first ? true : false;
    }

    function getFirst()
    {
        return (object) $this->first;
    }

    function hasNext()
    {
        return $this->row ? true : false;
    }

    function next()
    {
        $ret       = $this->row;
        $this->row = null;
        $this->row = mysqli_fetch_array( $this->result, MYSQLI_ASSOC );

        return (object) $ret;
    }
}

class Results
{
    protected $it;
    protected $array;

    function __construct( $anM, $aResult )
    {
        $this->it    = new ResultIterator( $anM, $aResult );
        $this->array = null;
    }

    function hasFirst()
    {
        return $this->it->hasFirst();
    }

    function getFirst()
    {
        return $this->it->getFirst();
    }


    //  Called by OutputResponse.TranslateToCSV
    function toArray()
    {
        if ( is_null( $this->array ) )
        {
            $this->array = array();

            while ( $this->it->hasNext() )
            {
                $this->array[] = (object) $this->it->next();
            }
        }

        return $this->array;
    }

    function toStringX()
    {
        return var_export( $this->array, true );
    }

    function isProcessed()
    {
        return !is_null( $this->array );
    }

    function encodeArrayX( $indent )
    {
        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "[";
        $sep = "";
        
        while ( $this->it->hasNext() )
        {
            $value = (object) $this->it->next();

            echo $sep;
            \JStream::EncodeValue( $indent + 1, $value ); 
            $sep = ",";
        }

        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "]";
    }

    function iterator()
    {
        return $this->it;
    }
}

function MySQLProcedureResults( $db, $sp_call, &$procedure = "" )
{
    $ret = False;

    if ( !is_object( $sp_call ) )
    {
        debug_backtrace();
    }

    $proc_name = $sp_call->proc_name;
    $arguments = $sp_call->arguments;

    $m = MySQLCreate();

    if ( ! mysqli_select_db( $m, $db ) )
    {
        NeoLog( "Aborting, could not select database! " . $db );
        MySQLClose( $m );
        exit;
    }
    else
    {
        $filtered = array();
        foreach( $arguments as $argument )
        {
            if ( is_null( $argument ) )
            {
                $filtered[] = "NULL";
            }
            else
            {
                $filtered[] = "'" . mysqli_real_escape_string( $m, $argument ) . "'";
            }
        }

        $procedure = $proc_name . "(" . join( ",", $filtered ) . ")";

        $sql_query = "CALL " . $procedure;

        if ( "wab_test" == $proc_name )
        {
            NeoLog( "Test: " . $sql_query );
        }

        $resource  = mysqli_query( $m, $sql_query, MYSQLI_USE_RESULT );

        NeoLog( "MySQL - MYSQLI_USE_RESULT" );

        if ( True === $resource )
        {
            $ret = array();
        }
        else
        if( False === $resource )
        {
            $ret = $m->error;
        }
        else
        if ( $resource )
        {
            $ret = new Results( $m, $resource );
        }
        else
        {
            NeoLog( "Aborting, " . $m->error );
            MySQLClose( $m );
            exit;
        }
    }

    if ( !is_a( $ret, 'Results' ) )
    {
        mysqli_next_result( $m );
        MySQLClose( $m );
    }

    return $ret;
}

